@@ -1,4 +1,5 @@
 """ Python Code for Communication with the Ecobee Thermostat """
+import datetime
 from typing import Optional
 
 import requests
@@ -16,15 +17,18 @@ from .const import (
     ECOBEE_API_VERSION,
     ECOBEE_AUTHORIZATION_CODE,
     ECOBEE_BASE_URL,
+    ECOBEE_AUTH_BASE_URL,
     ECOBEE_CONFIG_FILENAME,
     ECOBEE_DEFAULT_TIMEOUT,
     ECOBEE_ENDPOINT_AUTH,
     ECOBEE_ENDPOINT_THERMOSTAT,
     ECOBEE_ENDPOINT_TOKEN,
     ECOBEE_REFRESH_TOKEN,
+    ECOBEE_AUTH0_TOKEN,
     ECOBEE_OPTIONS_NOTIFICATIONS,
+    ECOBEE_WEB_CLIENT_ID
 )
-from .errors import ExpiredTokenError, InvalidTokenError
+from .errors import ExpiredTokenError, InvalidSensorError, InvalidTokenError
 from .util import config_from_file, convert_to_bool
 
 
@@ -40,6 +44,7 @@ class Ecobee(object):
         self.authorization_code = None
         self.access_token = None
         self.refresh_token = None
+        self.auth0_token = None
         self.include_notifications = False
 
         if self.config_filename is None and self.config is None:
@@ -55,6 +60,8 @@ class Ecobee(object):
                 self.authorization_code = self.config[ECOBEE_AUTHORIZATION_CODE]
             if ECOBEE_REFRESH_TOKEN in self.config:
                 self.refresh_token = self.config[ECOBEE_REFRESH_TOKEN]
+            if ECOBEE_AUTH0_TOKEN in self.config:
+                self.auth0_token = self.config[ECOBEE_AUTH0_TOKEN]
             if ECOBEE_OPTIONS_NOTIFICATIONS in self.config:
                 self.include_notifications = convert_to_bool(self.config[ECOBEE_OPTIONS_NOTIFICATIONS])
         else:
@@ -71,6 +78,8 @@ class Ecobee(object):
                 self.authorization_code = self.config[ECOBEE_AUTHORIZATION_CODE]
             if ECOBEE_REFRESH_TOKEN in self.config:
                 self.refresh_token = self.config[ECOBEE_REFRESH_TOKEN]
+            if ECOBEE_AUTH0_TOKEN in self.config:
+                self.auth0_token = self.config[ECOBEE_AUTH0_TOKEN]
             if ECOBEE_OPTIONS_NOTIFICATIONS in self.config:
                 self.include_notifications = convert_to_bool(self.config[ECOBEE_OPTIONS_NOTIFICATIONS])
 
@@ -80,6 +89,7 @@ class Ecobee(object):
         config[ECOBEE_API_KEY] = self.api_key
         config[ECOBEE_ACCESS_TOKEN] = self.access_token
         config[ECOBEE_REFRESH_TOKEN] = self.refresh_token
+        config[ECOBEE_AUTH0_TOKEN] = self.auth0_token
         config[ECOBEE_AUTHORIZATION_CODE] = self.authorization_code
         config[ECOBEE_OPTIONS_NOTIFICATIONS] = str(self.include_notifications)
         if self._file_based_config:
@@ -120,6 +130,9 @@ class Ecobee(object):
 
     def request_tokens(self) -> bool:
         """Requests API tokens from ecobee."""
+        if self.auth0_token is not None:
+            return self.request_tokens_web()
+        
         params = {
             "grant_type": "ecobeePin",
             "code": self.authorization_code,
@@ -146,8 +159,49 @@ class Ecobee(object):
         except (KeyError, TypeError) as err:
             _LOGGER.debug(f"Error obtaining tokens from ecobee: {err}")
             return False
+        
+    def request_tokens_web(self) -> bool:
+        assert self.auth0_token is not None, "auth0 token must be set before calling request_tokens_web"
+
+        resp = requests.get(ECOBEE_AUTH_BASE_URL + "/" + ECOBEE_ENDPOINT_AUTH, cookies={"auth0": self.auth0_token}, params={
+            "client_id": ECOBEE_WEB_CLIENT_ID,
+            "scope": "smartWrite",
+            "response_type": "token",
+            "response_mode": "form_post",
+            "redirect_uri": "https://www.ecobee.com/home/authCallback",
+            "audience": "https://prod.ecobee.com/api/v1",
+        }, timeout=ECOBEE_DEFAULT_TIMEOUT)
+
+        if resp.status_code != 200:
+            _LOGGER.error(f"Failed to refresh access token: {resp.status_code} {resp.text}")
+            return False
+        
+        if (auth0 := resp.cookies.get("auth0")) is None:
+            _LOGGER.error("Failed to refresh access token: no auth0 cookie in response")
+        self.auth0_token = auth0
+
+        # Parse the response HTML for the access token and expiration
+        if (access_token := resp.text.split('name="access_token" value="')[1].split('"')[0]) is None:
+            _LOGGER.error("Failed to refresh bearer token: no access token in response")
+            return False
+        
+        self.access_token = access_token
+
+        if (expires_in := resp.text.split('name="expires_in" value="')[1].split('"')[0]) is None:
+            _LOGGER.error("Failed to refresh bearer token: no expiration in response")
+            return False
+
+        expires_at = datetime.datetime.now() + datetime.timedelta(seconds=int(expires_in))
+        _LOGGER.debug(f"Access token expires at {expires_at}")
+
+        self._write_config()
+
+        return True
 
     def refresh_tokens(self) -> bool:
+        if self.auth0_token is not None:
+            return self.request_tokens_web()
+        
         """Refreshes ecobee API tokens."""
         params = {
             "grant_type": "refresh_token",
@@ -187,6 +241,7 @@ class Ecobee(object):
                 "includeEvents": "true",
                 "includeWeather": "true",
                 "includeSettings": "true",
+                "includeLocation": "true",
             }
         }
         if self.include_notifications:
@@ -211,7 +266,7 @@ class Ecobee(object):
     def get_remote_sensors(self, index: int) -> str:
         """Returns remote sensors from a thermostat based on list index of self.thermostats."""
         return self.thermostats[index]["remoteSensors"]
-    
+
     def get_equipment_notifications(self, index: int) -> str:
         """Returns equipment notifications from a thermostat based on list index of self.thermostats."""
         return self.thermostats[index]["notificationSettings"]["equipment"]
@@ -729,8 +784,14 @@ class Ecobee(object):
         except (ExpiredTokenError, InvalidTokenError) as err:
             raise err
 
-    def update_climate_sensors(self, index: int, climate_name: str, sensor_names: list) -> None:
-        """Get current climate program."""
+    def update_climate_sensors(self, index: int, climate_name: str, sensor_names: Optional[list]=None, sensor_ids: Optional[list]=None) -> None:
+        """Get current climate program. Must provide either `sensor_names` or `ids`."""
+        # Ensure only either `sensor_names` or `ids` was provided.
+        if sensor_names is None and sensor_ids is None:
+            raise ValueError("Need to provide either `sensor_names` or `ids`.")
+        if sensor_names and sensor_ids:
+            raise ValueError("Either `sensor_names` or `ids` should be provided, not both.")
+
         programs: dict = self.thermostats[index]["program"]
         # Remove currentClimateRef key.
         programs.pop("currentClimateRef", None)
@@ -738,15 +799,29 @@ class Ecobee(object):
         for i, climate in enumerate(programs["climates"]):
             if climate["name"] == climate_name:
                 climate_index = i
-        """Update climate sensors with sensor_names list."""
+        sensors = self.get_remote_sensors(index)
         sensor_list = []
-        for name in sensor_names:
-            """Find the sensor id from the name."""
-            sensors = self.get_remote_sensors(index)
-            for sensor in sensors:
-                if sensor["name"] == name:
-                    sensor_list.append(
-                        {"id": "{}:1".format(sensor["id"]), "name": name})
+
+        if sensor_ids:
+            """Update climate sensors with sensor_ids list."""
+            for id in sensor_ids:
+                for sensor in sensors:
+                    if sensor["id"] == id:
+                        sensor_list.append(
+                            {"id": "{}:1".format(id), "name": sensor["name"]})
+
+        if sensor_names:
+            """Update climate sensors with sensor_names list."""
+            for name in sensor_names:
+                """Find the sensor id from the name."""
+                for sensor in sensors:
+                    if sensor["name"] == name:
+                        sensor_list.append(
+                            {"id": "{}:1".format(sensor["id"]), "name": name})
+
+        if len(sensor_list) == 0:
+            raise InvalidSensorError("no sensor matching provided ids or names on thermostat")
+
         try:
             programs["climates"][climate_index]["sensors"] = sensor_list
         except UnboundLocalError:
@@ -848,7 +923,7 @@ class Ecobee(object):
             _LOGGER.debug(
                 f"Request response: {response.status_code}: {log_msg}"
             )
-            
+
             response.raise_for_status()
             return response.json()
         except HTTPError:
