@@ -7,6 +7,7 @@ ECOBEE_ACCESS_TOKEN = "ACCESS_TOKEN"
 ECOBEE_API_KEY = "API_KEY"
 ECOBEE_AUTHORIZATION_CODE = "AUTHORIZATION_CODE"
 ECOBEE_REFRESH_TOKEN = "REFRESH_TOKEN"
+ECOBEE_AUTH0_TOKEN = "AUTH0_TOKEN"
 
 ECOBEE_CONFIG_FILENAME = "ecobee.conf"
 
@@ -19,11 +20,14 @@ ECOBEE_STATE_CALIBRATING = -5003
 ECOBEE_VALUE_UNKNOWN = "unknown"
 
 ECOBEE_BASE_URL = "https://api.ecobee.com"
+ECOBEE_AUTH_BASE_URL = "https://auth.ecobee.com"
 ECOBEE_ENDPOINT_AUTH = "authorize"
 ECOBEE_ENDPOINT_TOKEN = "token"
 ECOBEE_ENDPOINT_THERMOSTAT = "thermostat"
 ECOBEE_API_VERSION = "1"
 
+ECOBEE_WEB_CLIENT_ID = "183eORFPlXyz9BbDZwqexHPBQoVjgadh"
+
 ECOBEE_MODEL_TO_NAME = {
     "idtSmart": "ecobee Smart Thermostat",
     "idtEms": "ecobee Smart EMS Thermostat",
